'use strict';

var assert = require( 'assert' );
var mqttRegexParser = require( '../' );

describe( 'parse', function() {

    describe( 'when a single level topic', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello' );
            assert( regex.exec( 'hello', 'should match topic' ) );

            assert.equal( '^hello$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when a multiple level topic', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello/world' );
            assert( regex.exec( 'hello/world', 'should match topic' ) );

            assert.equal( '^hello/world$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when topic contains escapable characters', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello.world' );
            assert( regex.exec( 'hello.world' ) );

            assert.equal( '^hello\\.world$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when topic contains numbers', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello123' );
            assert( regex.exec( 'hello123' ) );

            assert.equal( '^hello123$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when pattern with +', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello/+/my/+' );
            assert( regex.exec( 'hello/dog/my/friend' ), 'should match valid topics' );
            assert( !regex.exec( 'hello/dogmy/friend' ), 'should not bleed' );

            assert.equal( '^hello/[\\w\\.\\-\\$]+/my/[\\w\\.\\-\\$]+$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

    describe( 'when pattern with #', function() {
        it( 'should return the regex escaped form', function() {

            var regex = mqttRegexParser( 'hello/#' );
            assert( regex.exec( 'hello/dog/my/friend' ), 'should match valid topics' );
            assert( !regex.exec( 'hellosir' ), 'should not bleed' );

            assert.equal( '^hello/[\\w\\.\\-\\$]+(/[\\w\\.\\-\\$]+)*$', regex.source );
            assert.equal( false, regex.ignoreCase );
            assert.equal( false, regex.global );
        } );
    } );

} );
